import java.io.*;
import java.nio.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.*;
import java.net.*;

public class Connection implements Runnable {

    private DataInputStream dis;
    private DataOutputStream dos;
    private Socket socket;

    // konstruktor
    public Connection(Socket socket) throws IOException {
        this.dis = new DataInputStream(socket.getInputStream());
        this.dos = new DataOutputStream(socket.getOutputStream());
        this.socket = socket;
    }

    @Override
    public void run() {
        StringBuilder tc;

        while (true) {
            try {
                String outputMessage = "OK.";
                String filename;
                String received = "";
                int bytesRead = 0;
                boolean end = false;
                byte[] messageByte = new byte[1024];

                // wczytywanie wejscia
                int bytesToRead = this.dis.readInt();
                while (!end) {
                    bytesRead = this.dis.read(messageByte);
                    received += new String(messageByte, 0, bytesRead);

                    if (received.length() == bytesToRead) {
                        end = true;
                    }
                }

                // log
                System.out.println(socket + " Input: " + received);

                if (received.startsWith("/exit")) {
                    this.socket.close();
                    Server.connections.removeElement(this);
                    break;
                }

                if (received.startsWith("/send")) {
                    filename = Server.storageDir + "/" + received.substring(6).trim();
                    Path file = Paths.get(filename);
                    bytesToRead = this.dis.readInt();
                    System.out.println(this.socket + " wysyla plik "+filename+". Rozmiar: "+bytesToRead+" bajtow. ");

                    boolean first = true;
                    while (bytesToRead > 0) {
                        bytesRead = this.dis.read(messageByte);
                        if (first) {
                            first = false;
                            Files.write(file,Arrays.copyOf(messageByte, bytesRead));
                        } else {
                            Files.write(file, Arrays.copyOf(messageByte, bytesRead), StandardOpenOption.APPEND);
                        }
                        bytesToRead -= bytesRead;
                    }
                    System.out.println(this.socket + " Zapisano plik.");
                    outputMessage = "Zapisano plik!.";
                }

                if (received.startsWith("/ls")) {
                    File folder = new File(Server.storageDir);
                    File[] listOfFiles = folder.listFiles();
                    tc = new StringBuilder();
                    tc.append(String.format("Znaleziono %s plikow\n", listOfFiles.length));
                    for (File file : listOfFiles) {
                        if (file.isFile()) {
                            tc.append(file.getName()).append("\n");
                        }
                    }
                    outputMessage = tc.toString();
                }

                if (received.startsWith("/rm")) {
                    filename = received.substring(4).trim();
                    File file = new File(Server.storageDir + "/" + filename);

                    if (file.delete()) {
                        outputMessage = String.format("Usunieto plik %s ", filename);
                    } else {
                        outputMessage = String.format("Blad przy usunieciu pliku %s", filename);
                    }
                }

                this.dos.write(ByteBuffer.allocate(4).putInt(outputMessage.length()).array());
                this.dos.write(outputMessage.getBytes());

            } catch (IOException e) {
                e.printStackTrace();

                break;
            }

        }

        try {
            // closing resources
            this.dis.close();
            this.dos.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
